
/* Map-specific definitions. You may think of it as
   an extension for the ruleset.

   Feel free to redefine everything in this file to
   adapt this script to your map.

   The only thing this module should do is provide
   a function called userInit(), without arguments,
   that would be called once at game start.

   This function, apart from any map-specific initialization,
   should set a timer on another function that would
   call creepsSend() from creeps module repeatedly with
   different arguments to actually send in the creeps.
*/

var stats = {
	weapons: [
		[
			{ stat: "MG1Mk1", weight: WEIGHT.ULTRALIGHT, value: 5 },
			{ stat: "MG2Mk1", weight: WEIGHT.LIGHT, value: 10 },
			{ stat: "MG3Mk1", weight: WEIGHT.MEDIUM, value: 15 },
			{ stat: "MG4ROTARYMk1", weight: WEIGHT.MEDIUM, value: 25 },
			{ stat: "MG5TWINROTARY", weight: WEIGHT.MEDIUM, value: 35 },
			{ stat: "Laser3BEAMMk1", weight: WEIGHT.ULTRALIGHT, value: 50 },
			{ stat: "Laser2PULSEMk1", weight: WEIGHT.HEAVY, value: 75 },
			{ stat: "HeavyLaser", weight: WEIGHT.ULTRAHEAVY, value: 100 },
		],

		[
			{ stat: "Cannon1Mk1", weight: WEIGHT.LIGHT, value: 15 },
			{ stat: "Cannon2A-TMk1", weight: WEIGHT.HEAVY, value: 20 },
			{ stat: "Cannon4AUTOMk1", weight: WEIGHT.HEAVY, value: 30 }, 
			{ stat: "Cannon5VulcanMk1", weight: WEIGHT.HEAVY, value: 40 },
			{ stat: "Cannon6TwinAslt", weight: WEIGHT.ULTRAHEAVY, value: 50 },
			{ stat: "Cannon375mmMk1", weight: WEIGHT.ULTRAHEAVY, value: 60 },
			{ stat: "RailGun1Mk1", weight: WEIGHT.LIGHT, value: 75 },
			{ stat: "RailGun2Mk1", weight: WEIGHT.HEAVY, value: 125 },
			{ stat: "RailGun3Mk1", weight: WEIGHT.ULTRAHEAVY, value: 150 }, 
		],

		[
			{ stat: "Rocket-Pod", weight: WEIGHT.LIGHT, value: 15 },
			{ stat: "Rocket-LtA-T", weight: WEIGHT.LIGHT, value: 20 },
			{ stat: "Rocket-MRL", weight: WEIGHT.LIGHT, value: 25 },
			{ stat: "Rocket-IDF", weight: WEIGHT.ULTRAHEAVY, value: 35 },
			{ stat: "Rocket-HvyA-T", weight: WEIGHT.LIGHT, value: 45 },
			{ stat: "Missile-A-T", weight: WEIGHT.LIGHT, value: 65 },
			{ stat: "Missile-MdArt", weight: WEIGHT.HEAVY, value: 90 },
			{ stat: "Missile-HvyArt", weight: WEIGHT.ULTRAHEAVY, value: 125 },
		],

		[
			{ stat: "Flame1Mk1", weight: WEIGHT.LIGHT, value: 10 },
			{ stat: "Flame2", weight: WEIGHT.HEAVY, value: 25 },
			{ stat: "PlasmiteFlamer", weight: WEIGHT.HEAVY, value: 40 },
			{ stat: "Laser3BEAMMk1", weight: WEIGHT.ULTRALIGHT, value: 50 },
			{ stat: "Laser2PULSEMk1", weight: WEIGHT.HEAVY, value: 75 },
			{ stat: "HeavyLaser", weight: WEIGHT.ULTRAHEAVY, value: 100 },
		],

		[
			{ stat: "Mortar1Mk1", weight: WEIGHT.HEAVY, value: 25 },
			{ stat: "Mortar2Mk1", weight: WEIGHT.HEAVY, value: 30 },
			{ stat: "Mortar3ROTARYMk1", weight: WEIGHT.HEAVY, value: 35 },
			{ stat: "Howitzer105Mk1", weight: WEIGHT.ULTRAHEAVY, value: 45 },
			{ stat: "Mortar-Incenediary", weight: WEIGHT.HEAVY, value: 50 },
			{ stat: "Howitzer03-Rot", weight: WEIGHT.ULTRAHEAVY, value: 60 },
			{ stat: "Howitzer150Mk1", weight: WEIGHT.ULTRAHEAVY, value: 75 },
			{ stat: "Howitzer-Incenediary", weight: WEIGHT.ULTRAHEAVY, value: 100 },
		],
	],
	bodies: [
		[
			{ stat: "Body4ABT", weight: WEIGHT.LIGHT, value: 5 },
			{ stat: "Body8MBT", weight: WEIGHT.MEDIUM, value: 25 },
			{ stat: "Body12SUP", weight: WEIGHT.HEAVY, value: 45 },
		],

		[
			{ stat: "Body2SUP", weight: WEIGHT.LIGHT, value: 10 },
			{ stat: "Body6SUPP", weight: WEIGHT.MEDIUM, value: 30 },
			{ stat: "Body9REC", weight: WEIGHT.HEAVY, value: 60 },
		],

		[
			{ stat: "Body3MBT", weight: WEIGHT.LIGHT, value: 20 },
			{ stat: "Body7ABT", weight: WEIGHT.MEDIUM, value: 45 },
			{ stat: "Body10MBT", weight: WEIGHT.HEAVY, value: 80 },
		],
	],
	propulsions: [
		[
			{ stat: "wheeled01", value: 5 },
			{ stat: "HalfTrack", value: 15 },
			{ stat: "tracked01", value: 25 },
		],

		[
			{ stat: "hover01", value: 30 },
		],
	],
	cyborgs32: [
		{ body: "CyborgLightBody", propulsion: "CyborgLegs", weapons: [ "CyborgChaingun", ], value: 30 }, // mg cyborg
		{ body: "CyborgLightBody", propulsion: "CyborgLegs", weapons: [ "CyborgFlamer01", ], value: 35 }, // flamer cyborg
		{ body: "CyborgLightBody", propulsion: "CyborgLegs", weapons: [ "CyborgCannon", ], value: 45 }, // lc borg
		{ body: "CyborgLightBody", propulsion: "CyborgLegs", weapons: [ "Cyb-Wpn-Grenade", ], value: 55 }, // mortar borg
		{ body: "CyborgLightBody", propulsion: "CyborgLegs", weapons: [ "CyborgRocket", ], value: 60 }, // lancer borg
		{ body: "CyborgLightBody", propulsion: "CyborgLegs", weapons: [ "Cyb-Wpn-Thermite", ], value: 65 }, // thermite cyborg
		{ body: "CyborgLightBody", propulsion: "CyborgLegs", weapons: [ "CyborgRotMG", ], value: 75 }, // ag cyborg
		{ body: "CyborgHeavyBody", propulsion: "CyborgLegs", weapons: [ "Cyb-Hvywpn-Mcannon", ], value: 90 }, // mc super
		{ body: "CyborgHeavyBody", propulsion: "CyborgLegs", weapons: [ "Cyb-Hvywpn-HPV", ], value: 95 }, // hpv super
		{ body: "CyborgHeavyBody", propulsion: "CyborgLegs", weapons: [ "Cyb-Hvywpn-TK", ], value: 100 }, // tk super
		{ body: "CyborgHeavyBody", propulsion: "CyborgLegs", weapons: [ "Cyb-Hvywpn-Acannon", ], value: 110 }, // ac super
		{ body: "CyborgLightBody", propulsion: "CyborgLegs", weapons: [ "Cyb-Wpn-Laser", ], value: 120 }, // flash borg
		{ body: "CyborgLightBody", propulsion: "CyborgLegs", weapons: [ "Cyb-Wpn-Atmiss", ], value: 140 }, // scourge borg
		{ body: "CyborgLightBody", propulsion: "CyborgLegs", weapons: [ "Cyb-Wpn-Rail1", ], value: 160 }, // needle borg
		{ body: "CyborgHeavyBody", propulsion: "CyborgLegs", weapons: [ "Cyb-Hvywpn-PulseLsr", ], value: 200 }, // pulse super
		{ body: "CyborgHeavyBody", propulsion: "CyborgLegs", weapons: [ "Cyb-Hvywpn-A-T", ], value: 250 }, // scourge super
		{ body: "CyborgHeavyBody", propulsion: "CyborgLegs", weapons: [ "Cyb-Hvywpn-RailGunner", ], value: 300 }, // rail super
	],
	cyborgs31: [
		{ body: "CyborgChain1Ground", propulsion: "CyborgLegs", weapons: [ "CyborgChaingun", ], value: 30 }, // mg cyborg
		{ body: "CyborgFlamerGrd", propulsion: "CyborgLegs", weapons: [ "CyborgFlamer01", ], value: 35 }, // flamer cyborg
		{ body: "CyborgCannonGrd", propulsion: "CyborgLegs", weapons: [ "CyborgCannon", ], value: 45 }, // lc borg
		{ body: "Cyb-Bod-Grenade", propulsion: "CyborgLegs", weapons: [ "Cyb-Wpn-Grenade", ], value: 55 }, // mortar borg
		{ body: "CyborgRkt1Ground", propulsion: "CyborgLegs", weapons: [ "CyborgRocket", ], value: 60 }, // lancer borg
		{ body: "Cyb-Bod-Thermite", propulsion: "CyborgLegs", weapons: [ "Cyb-Wpn-Thermite", ], value: 65 }, // thermite cyborg
		{ body: "CybRotMgGrd", propulsion: "CyborgLegs", weapons: [ "CyborgRotMG", ], value: 75 }, // ag cyborg
		{ body: "Cyb-Hvybod-Mcannon", propulsion: "CyborgLegs", weapons: [ "Cyb-Hvywpn-Mcannon", ], value: 90 }, // mc super
		{ body: "Cyb-Hvybod-HPV", propulsion: "CyborgLegs", weapons: [ "Cyb-Hvywpn-HPV", ], value: 95 }, // hpv super
		{ body: "Cyb-Hvybod-TK", propulsion: "CyborgLegs", weapons: [ "Cyb-Hvywpn-TK", ], value: 100 }, // tk super
		{ body: "Cyb-Hvybod-Acannon", propulsion: "CyborgLegs", weapons: [ "Cyb-Hvywpn-Acannon", ], value: 110 }, // ac super
		{ body: "Cyb-Bod-Las1", propulsion: "CyborgLegs", weapons: [ "Cyb-Wpn-Laser", ], value: 120 }, // flash borg
		{ body: "Cyb-Bod-Atmiss", propulsion: "CyborgLegs", weapons: [ "Cyb-Wpn-Atmiss", ], value: 140 }, // scourge borg
		{ body: "Cyb-Bod-Rail1", propulsion: "CyborgLegs", weapons: [ "Cyb-Wpn-Rail1", ], value: 160 }, // needle borg
		{ body: "Cyb-Hvybod-PulseLsr", propulsion: "CyborgLegs", weapons: [ "Cyb-Hvywpn-PulseLsr", ], value: 200 }, // pulse super
		{ body: "Cyb-Hvybod-A-T", propulsion: "CyborgLegs", weapons: [ "Cyb-Hvywpn-A-T", ], value: 250 }, // scourge super
		{ body: "Cyb-Hvybod-RailGunner", propulsion: "CyborgLegs", weapons: [ "Cyb-Hvywpn-RailGunner", ], value: 300 }, // rail super
	],
};

/* Armor upgrades for creeps */

var creepResearchStatus = 0;
var creepResearch = [];
for (var i = 1; i <= 9; ++i) {
	creepResearch = creepResearch.concat([
		"R-Vehicle-Armor-Heat0" + i,
		"R-Vehicle-Metals0" + i,
		"R-Cyborg-Armor-Heat0" + i,
		"R-Cyborg-Metals0" + i,
	]);
}

function userCreepUpgrades() {
	if (creepResearchStatus >= creepResearch.length) {
		removeTimer("userCreepUpgrades");
		return;
	}
	for (var i = 1; i <= 3; ++i) {
    hackNetOff();
		completeResearch(creepResearch[creepResearchStatus], i);
    hackNetOn();
  }
	++creepResearchStatus;
}



/* We're keeping track of how many times we tried to send
   a wave of creeps and how many times we actually managed
   to do that without running out of our budget.

   This is a way of providing a deterministic, yet
   not too much predictable series of waves.
*/

// number of waves successfully sent
var waveNumber = 0;

// number of attempts to send a wave
var waveSeed = -1;

// user initialization function, called by the events module at game start
function userInit() {

	waveNumber = 0;
	waveSeed = 0;

	// set timeout to send in creeps
	queue("userTimer", 10000);

	// add one armor upgrade to all creeps every 5 minutes
	setTimer("userCreepUpgrades", 75000);

}

// creep wave design timer function
function userTimer() {

	++waveSeed;

	var player = waveSeed % 3 + 1;

	var position = [
		{ /* human */ },
		{ x:  1, y: 8 },
		{ x: 13, y: 1 },
		{ x: 35, y: 1 }
	][player];

	var caption = [
		"New Paradigm",
		"Collective",
		"NEXUS",
	][player - 1];

	// both creep count and single creep strength grows linearly in this scenario
	var bossWave = (waveNumber % 10 == 9);
	var creepCount = Math.floor(3 + waveNumber / 5);
	var totalValue = 60 + waveNumber * waveSeed / 10;
	if (bossWave) {
		caption = "heavy " + caption;
		creepCount = Math.floor(waveNumber / 10);
		totalValue= 60 + waveNumber * waveNumber / 10;
	}


	var waveType = waveSeed % 7;
	var bodies, propulsions, weapons, templates;

	switch (waveType) {

		case 4: // cyborgs
			if (version.indexOf("3.1") > -1)
				templates = stats.cyborgs31;
			else
				templates = stats.cyborgs32;
			caption += " cyborgs";
			break;

		case 6: // hovers
			bodies = stats.bodies[player - 1];
			propulsions = stats.propulsions[1];
			weapons = stats.weapons[waveSeed % stats.weapons.length];
			caption += " hovercrafts";
			break;

		default: // tanks
			bodies = stats.bodies[player - 1];
			propulsions = stats.propulsions[0];
			weapons = stats.weapons[waveSeed % stats.weapons.length];
			caption += " tanks";
			break;
	}

	var totalBounty = 100 + waveNumber;
	var cnt = 0;

	if (waveType != 4) {

		cnt = creepsSpawnSmart(
			player,
			position,
			creepCount,
			totalValue,
			totalBounty,
			caption,
			bodies,
			propulsions,
			weapons
		);

	} else {

		// creepsSpawnSmart() does not support cyborgs yet,
		// so we shall do it manually

		var idx = -1;
		for (var i = 0; i < templates.length; ++i) {
			if (templates[i].value * creepCount < totalValue)
				idx = i;
		}

		if (idx > -1) {
			creepsSpawn(
				player,
				position,
				creepCount,
				totalBounty / creepCount,
				caption,
				templates[idx].body,
				templates[idx].propulsion,
				templates[idx].weapons[0]
			);
			cnt = creepCount;
		}

	}

	// check if wave was successfully sent
	if (cnt > 0) {
		++waveNumber;
		queue("userTimer", 30000);
	} else {
		queue("userTimer", 100);
	}

}
